@@ -14,7 +14,6 @@ from pathlib import Path
 from tempfile import gettempdir
 from typing import Any
 
-import click
 import cloup
 import pytest
 import requests
@@ -34,7 +33,7 @@ LOGGER = get_logger(name="class_generator")
 TESTS_MANIFESTS_DIR: str = "class_generator/tests/manifests"
 SCHEMA_DIR: str = "class_generator/schema"
 RESOURCES_MAPPING_FILE: str = os.path.join(SCHEMA_DIR, "__resources-mappings.json")
-MISSING_DESCRIPTION_STR: str = "No field description from API; please add description"
+MISSING_DESCRIPTION_STR: str = "No field description from API"
 
 
 def _is_kind_and_namespaced(
@@ -155,13 +154,16 @@ def get_server_version(client: str):
 
 
 def get_client_binary() -> str:
-    if os.system("which oc") == 0:
+    if shutil.which("oc"):
+        LOGGER.debug("Using 'oc' client.")
         return "oc"
 
-    elif os.system("which kubectl") == 0:
+    elif shutil.which("kubectl"):
+        LOGGER.debug("Using 'kubectl' client.")
         return "kubectl"
+
     else:
-        LOGGER.error("Failed to find oc or kubectl")
+        LOGGER.error("Failed to find 'oc' or 'kubectl' in PATH.")
         sys.exit(1)
 
 
@@ -204,9 +206,9 @@ def update_kind_schema():
     cluster_version = cluster_version.split("+")[0]
     ocp_openapi_json_file = Path(gettempdir()) / f"__k8s-openapi-{cluster_version}__.json"
 
-    newer_version: bool = Version(cluster_version) > Version(last_cluster_version_generated)
+    same_or_newer_version: bool = Version(cluster_version) >= Version(last_cluster_version_generated)
 
-    if newer_version:
+    if same_or_newer_version:
         with open(cluster_version_file, "w") as fd:
             fd.write(cluster_version)
 
@@ -219,7 +221,7 @@ def update_kind_schema():
         LOGGER.error("Failed to generate schema.")
         sys.exit(1)
 
-    if newer_version:
+    if same_or_newer_version:
         # copy all files from tmp_schema_dir to schema dir
         shutil.copytree(src=tmp_schema_dir, dst=SCHEMA_DIR, dirs_exist_ok=True)
 
@@ -236,16 +238,24 @@ def update_kind_schema():
                     sys.exit(1)
 
     map_kind_to_namespaced(
-        client=client, newer_cluster_version=newer_version, schema_definition_file=ocp_openapi_json_file
+        client=client, newer_cluster_version=same_or_newer_version, schema_definition_file=ocp_openapi_json_file
     )
 
+    # Copy the resources mapping file to fake_kubernetes_client for the fake client to use
+    fake_client_mappings = Path("fake_kubernetes_client/__resources-mappings.json")
+    try:
+        shutil.copy2(RESOURCES_MAPPING_FILE, fake_client_mappings)
+    except (OSError, IOError):
+        # Don't fail the entire process if copy fails
+        pass
 
-def convert_camel_case_to_snake_case(string_: str) -> str:
+
+def convert_camel_case_to_snake_case(name: str) -> str:
     """
     Converts a camel case string to snake case.
 
     Args:
-        string_ (str): The camel case string to convert.
+        name (str): The camel case string to convert.
 
     Returns:
         str: The snake case representation of the input string.
@@ -265,19 +275,20 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
         - The function handles both single-word camel case strings (e.g., "Service") and multi-word camel case strings
           (e.g., "myCamelCaseString").
     """
-    do_not_proccess_list = ["OAuth", "KubeVirt"]
+    do_not_process_list = ["oauth", "kubevirt"]
+
     # If the input string is in the do_not_proccess_list, return it as it is.
-    if string_.lower() in [_str.lower() for _str in do_not_proccess_list]:
-        return string_.lower()
+    if name.lower() in do_not_process_list:
+        return name.lower()
 
     formatted_str: str = ""
 
-    if string_.islower():
-        return string_
+    if name.islower():
+        return name
 
     # For single words, e.g "Service" or "SERVICE"
-    if string_.istitle() or string_.isupper():
-        return string_.lower()
+    if name.istitle() or name.isupper():
+        return name.lower()
 
     # To decide if underscore is needed before a char, keep the last char format.
     # If previous char is uppercase, underscode should not be added. Also applied for the first char in the string.
@@ -287,9 +298,9 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
     # last word. Underscore should be added before it and all chars from here should be lowercase.
     following_capital_chars: re.Match | None = None
 
-    str_len_for_idx_check = len(string_) - 1
+    str_len_for_idx_check = len(name) - 1
 
-    for idx, char in enumerate(string_):
+    for idx, char in enumerate(name):
         # If lower case, append to formatted string
         if char.islower():
             formatted_str += char
@@ -302,15 +313,15 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
 
         else:
             if idx < str_len_for_idx_check:
-                following_capital_chars = re.search(r"[A-Z]", "".join(string_[idx + 1 :]))
+                following_capital_chars = re.search(r"[A-Z]", "".join(name[idx + 1 :]))
             if last_capital_char:
-                if idx < str_len_for_idx_check and string_[idx + 1].islower():
+                if idx < str_len_for_idx_check and name[idx + 1].islower():
                     if following_capital_chars:
                         formatted_str += f"_{char.lower()}"
                         last_capital_char = True
                         continue
 
-                    remaining_str = "".join(string_[idx:])
+                    remaining_str = "".join(name[idx:])
                     # The 2 letters in the string; uppercase char followed by lowercase char.
                     # Example: `clusterIPs`, handle `Ps` at this point
                     if idx + 1 == str_len_for_idx_check:
@@ -338,25 +349,6 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
     return formatted_str
 
 
-def render_jinja_template(template_dict: dict[Any, Any], template_dir: str, template_name: str) -> str:
-    env = Environment(
-        loader=FileSystemLoader(template_dir),
-        trim_blocks=True,
-        lstrip_blocks=True,
-        undefined=DebugUndefined,
-    )
-
-    template = env.get_template(name=template_name)
-    rendered = template.render(template_dict)
-    undefined_variables = meta.find_undeclared_variables(env.parse(rendered))
-
-    if undefined_variables:
-        LOGGER.error(f"The following variables are undefined: {undefined_variables}")
-        sys.exit(1)
-
-    return rendered
-
-
 def parse_user_code_from_file(file_path: str) -> tuple[str, str]:
     with open(file_path) as fd:
         data = fd.read()
@@ -385,6 +377,25 @@ def parse_user_code_from_file(file_path: str) -> tuple[str, str]:
     return user_code, user_imports
 
 
+def render_jinja_template(template_dict: dict[Any, Any], template_dir: str, template_name: str) -> str:
+    env = Environment(
+        loader=FileSystemLoader(template_dir),
+        trim_blocks=True,
+        lstrip_blocks=True,
+        undefined=DebugUndefined,
+    )
+
+    template = env.get_template(name=template_name)
+    rendered = template.render(template_dict)
+    undefined_variables = meta.find_undeclared_variables(env.parse(rendered))
+
+    if undefined_variables:
+        LOGGER.error(f"The following variables are undefined: {undefined_variables}")
+        sys.exit(1)
+
+    return rendered
+
+
 def generate_resource_file_from_dict(
     resource_dict: dict[str, Any],
     overwrite: bool = False,
@@ -405,7 +416,7 @@ def generate_resource_file_from_dict(
     )
 
     output = "# Generated using https://github.com/RedHatQE/openshift-python-wrapper/blob/main/scripts/resource/README.md\n\nfrom __future__ import annotations\n"
-    formatted_kind_str = convert_camel_case_to_snake_case(string_=resource_dict["kind"])
+    formatted_kind_str = convert_camel_case_to_snake_case(name=resource_dict["kind"])
     _file_suffix: str = f"{'_' + output_file_suffix if output_file_suffix else ''}"
 
     if add_tests:
@@ -516,7 +527,7 @@ def prepare_property_dict(
 
         val_schema = get_property_schema(property_=val)
         type_dict = types_generator(key_dict=val_schema)
-        python_name = convert_camel_case_to_snake_case(string_=f"{dict_key}_{key}" if key in keys_to_rename else key)
+        python_name = convert_camel_case_to_snake_case(name=f"{dict_key}_{key}" if key in keys_to_rename else key)
         resource_dict[dict_key].append({
             "name-for-class-arg": python_name,
             "property-name": key,
@@ -623,6 +634,7 @@ def class_generator(
     output_dir: str = "",
     add_tests: bool = False,
     called_from_cli: bool = True,
+    update_schema_executed: bool = False,
 ) -> list[str]:
     """
     Generates a class for a given Kind.
@@ -630,12 +642,34 @@ def class_generator(
     LOGGER.info(f"Generating class for {kind}")
     kind = kind.lower()
     kind_and_namespaced_mappings = read_resources_mapping_file().get(kind)
-
     if not kind_and_namespaced_mappings:
-        LOGGER.error(f"{kind} not found in {RESOURCES_MAPPING_FILE}, Please run with --update-schema")
         if called_from_cli:
-            sys.exit(1)
+            if update_schema_executed:
+                LOGGER.error(f"{kind} not found in {RESOURCES_MAPPING_FILE} after update-schema executed.")
+                sys.exit(1)
+
+            run_update_schema = input(
+                f"{kind} not found in {RESOURCES_MAPPING_FILE}, Do you want to run --update-schema and retry? [Y/N]"
+            )
+            if run_update_schema.lower() == "n":
+                sys.exit(1)
+
+            elif run_update_schema.lower() == "y":
+                update_kind_schema()
+
+                return class_generator(
+                    overwrite=overwrite,
+                    dry_run=dry_run,
+                    kind=kind,
+                    output_file=output_file,
+                    output_dir=output_dir,
+                    add_tests=add_tests,
+                    called_from_cli=True,
+                    update_schema_executed=True,
+                )
+
         else:
+            LOGGER.error(f"{kind} not found in {RESOURCES_MAPPING_FILE}, Please run --update-schema.")
             return []
 
     resources = parse_explain(kind=kind)
@@ -717,7 +751,7 @@ def generate_class_generator_tests() -> None:
 @cloup.option(
     "-k",
     "--kind",
-    type=click.STRING,
+    type=cloup.STRING,
     help="""
     \b
     The Kind to generate the class for, Needs working cluster with admin privileges.
@@ -729,7 +763,7 @@ def generate_class_generator_tests() -> None:
     "-o",
     "--output-file",
     help="The full filename path to generate a python resource file. If not sent, resource kind will be used",
-    type=click.Path(),
+    type=cloup.Path(),
 )
 @cloup.option(
     "--overwrite",
